@@ -4,15 +4,14 @@ import asyncio
 import time
 from functools import partial
 from types import SimpleNamespace
-from typing import Type, Generic, Callable, Dict, Any
+from typing import Type, Generic, Callable, Dict, Any, List
 
-import loguru
 import orjson
 from sanic import Sanic, Request
 from sanic.response import BaseHTTPResponse
 
+from web_foundation import settings
 from web_foundation.config import AppConf
-from web_foundation.extentions.addons.addon_loader import AddonsLoader
 from web_foundation.extentions.error_handler import ExtendedErrorHandler
 from web_foundation.extentions.metrics.events import NewMetricEvent
 from web_foundation.extentions.metrics.metric import HistogramMetric, CounterMetric, SummaryMetric, Metric
@@ -20,11 +19,11 @@ from web_foundation.extentions.realtime.broadcaster import RtBroadcaster
 from web_foundation.extentions.realtime.connection import RtEventCallback
 from web_foundation.extentions.router import ExtRouter
 from web_foundation.extentions.sanic_ext_worker import ExtWorker
-from web_foundation.kernel.messaging.channel import IChannel, IMessage, GenericIMessage
 from web_foundation.kernel.container import GenericDependencyContainer
-from web_foundation.kernel.messaging.dispatcher import IDispatcher, IDispatcherIsolate
 from web_foundation.kernel.isolate import Isolate
-from web_foundation import settings
+from web_foundation.kernel.messaging.channel import IChannel, IMessage, GenericIMessage
+from web_foundation.kernel.messaging.dispatcher import IDispatcher, IDispatcherIsolate
+from web_foundation.kernel.stage import AppStage, StageListener
 from web_foundation.settings_event import SettingsChangeEvent
 
 
@@ -34,11 +33,12 @@ class App(Generic[GenericDependencyContainer]):
     sanic: Sanic
     config: AppConf
     dispatcher: IDispatcher
-    # listeners: Dict[str, List[AppListener]]
     _rt_listeners = list[tuple[list[Type[IMessage] | str], RtEventCallback, bool]]
+    _stage_listeners: Dict[AppStage, List[StageListener]]
 
     def __init__(self, dependency_container: GenericDependencyContainer,
                  dispatcher: IDispatcher = None):
+        self.config = dependency_container.app_config()
         self.name = dependency_container.app_config().app_name
         self.container = dependency_container
         self.sanic = Sanic(self.name, loads=orjson.loads, dumps=orjson.dumps,
@@ -46,6 +46,8 @@ class App(Generic[GenericDependencyContainer]):
 
         self.dispatcher = dispatcher if dispatcher else IDispatcher()
         self._rt_listeners = []
+        self._stage_listeners = {}
+        self.container.set_listeners(self.listeners_registrator)
         self.add_isolate("Dispatcher", IDispatcherIsolate, dispatcher=self.dispatcher, add_channel=False, daemon=True)
 
     def add_isolate(self, name: str, isolate_cls: Type[Isolate], daemon=True, add_channel: bool = True, **kwargs):
@@ -85,11 +87,22 @@ class App(Generic[GenericDependencyContainer]):
         """
         self._rt_listeners.append((event_type, resolve_callback, use_nested_classes))
 
+    def listeners_registrator(self, stage: AppStage, callback: StageListener):
+        if not self._stage_listeners.get(stage):
+            self._stage_listeners[stage] = []
+        self._stage_listeners[stage].append(callback)
+
     def add_custom_router(self, router: ExtRouter):
         router.ctx.app = self.sanic
         self.sanic.router = router
         self.sanic.router.apply_routes(self.sanic, self.container)
 
+    async def _dispatch_stage(self, stage: AppStage):
+        if not stage in self._stage_listeners:
+            return
+        for listener in self._stage_listeners[stage]:
+            await listener(stage, self.container)
+
     def _set_sanic_confs(self):
         self.sanic.config.SWAGGER_UI_CONFIGURATION = {
             "docExpansion": 'none'
@@ -114,9 +127,6 @@ class App(Generic[GenericDependencyContainer]):
             s_app.ctx.container = self.container
             for service, name in s_app.ctx.container.services():
                 service.channel = s_app.ctx.channel
-            # add addon loader
-
-            # add broadcaster
             s_app.ctx.rt_broadcaster = RtBroadcaster()
             for event, resolver, use_nested_classes in self._rt_listeners:
                 s_app.ctx.channel.add_event_listener(
@@ -124,8 +134,8 @@ class App(Generic[GenericDependencyContainer]):
                     callback=partial(s_app.ctx.rt_broadcaster.broadcast, resolve_callback=resolver),
                     use_nested_classes=use_nested_classes)
             await self.container.init_resources(s_app.ctx.channel)
-            if hasattr(self.container, "addon_loader") and self.container.addon_loader.initialized:
-                await self.container.addon_loader().first_init()
+            if loader := self.container.addon_loader():
+                await loader.first_init()
 
         async def reg_metrics(s_app: Sanic):
             async def give_metric(metric: Metric):
@@ -181,22 +191,12 @@ class App(Generic[GenericDependencyContainer]):
 
         self.sanic.before_server_stop(on_stop)
 
-    # def _add_listener(self, scope: str, listener: AppListener):
-    #     if not self.listeners.get(scope):
-    #         self.listeners[scope] = []
-    #     self.listeners[scope].append(listener)
-    #
-    # def before_resources_init(self, listener: AppListener):
-    #     self._add_listener("before_resource", listener)
-
-    async def before_app_run(self):
-        await self.container.before_app_run()
-
     def run(self):
-        asyncio.run(self.before_app_run())
+        asyncio.run(self._dispatch_stage(AppStage.BEFORE_APP_RUN))
         self._set_sanic_confs()
         self._configure_sanic_main()
         self._configure_sanic_worker()
+        asyncio.run(self._dispatch_stage(AppStage.BEFORE_SERVER_START))
         sanic_conf = self.container.app_config().sanic
         start_args = sanic_conf.dict() if sanic_conf else {}
         self.sanic.run(**start_args)
