@@ -17,8 +17,8 @@ class Dependency(Generic[T]):  # type: ignore
         self._initialized = self._initialized
 
     def __call__(self, *args, **kwargs) -> T:
-        if not self._initialized and not self.default:
-            raise AttributeError("Dependency must me initialized with from_initialized() or with default kwarg ")
+        # if not self._initialized and not self.default:
+        #     raise AttributeError("Dependency must me initialized with from_initialized() or with default kwarg")
         if not self._initialized:
             return self.default
         return self._initialized
